#pragma once

#include "defs.h"
#include "utils.h"

S_BEGIN_DECLS

/**
 * @file
 * @defgroup BaseN BaseN
 * @addtogroup BaseN
 * @{
 * https://tools.ietf.org/html/rfc4648
 */

/**
 * Encode a Byte array to a Base16 string.
 */
S_EXPORTED
schar *
s_base_16_enc (const sbyte * input_data,
               size_t in_len,
               size_t * out_len);

/**
 * Decode a base16 encoded string into a Byte array.
 */
S_EXPORTED
sbyte *
s_base_16_dec (const schar * base16_str,
               size_t in_len,
               size_t * out_len);

/**
 * Encode a Byte array to a Base16 string.
 */
S_EXPORTED
schar *
s_base_32_enc (const sbyte * input_data,
               size_t in_len,
               size_t * out_len);

/**
 * Decode a base32 encoded string into a Byte array.
 */
sbyte *
s_base_32_dec (const schar * base32_str,
               size_t in_len,
               size_t * out_len);

/**
 * Encode a Byte array to a Base32 Hex (Extended hex) string.
 */
S_EXPORTED
schar *
s_base_32_hex_enc (const sbyte * input_data,
                   size_t in_len,
                   size_t * out_len);

/**
 * Decode a base32 Hex (Extended hex) encoded string into a Byte array.
 */
S_EXPORTED
sbyte *
s_base_32_hex_dec (const schar * base32_str,
                   size_t in_len,
                   size_t * out_len);

/**
 * Encode a Byte array to a Base64 string.
 */
S_EXPORTED
schar *
s_base_64_enc (const sbyte * input_data,
               size_t in_len,
               size_t * out_len);

/**
 * Decode a base64 encoded string into a Byte array.
 */
S_EXPORTED
sbyte *
s_base_64_dec (const schar * base64_str,
               size_t in_len,
               size_t * out_len);

/**
 * Encode a byte array into a URI safe Base64 string.
 */
S_EXPORTED
schar *
s_base_64_safe_enc (const sbyte * input_data,
                    size_t in_len,
                    size_t * out_len);

/**
 * Decode a URI safe Base64 string into a byte array.
 */
S_EXPORTED
sbyte *
s_base_64_safe_dec (const schar * base64_str,
                    size_t in_len,
                    size_t * out_len);



/**
 * @}
 */

S_END_DECLS
